#include <MIDI.h>
#include <midi_Defs.h>
#include <midi_Message.h>
#include <midi_Namespace.h>
#include <midi_Settings.h>

// チャンネル
#define CH 1
// ベロシティ
#define VELOCITY 100

#define NUM_OF_BUTTON 8
#define NUM_OF_MODE 3
#define ON 0
#define MODE_CONTROLLER 0
#define MODE_NUMPAD1 1
#define MODE_NUMPAD2 2

#define BUTTON_CONTROLLER_RIGHT 0
#define BUTTON_CONTROLLER_LEFT 1
#define BUTTON_CONTROLLER_DOWN 2
#define BUTTON_CONTROLLER_UP 3
#define BUTTON_CONTROLLER_START 4
#define BUTTON_CONTROLLER_SELECT 5
#define BUTTON_CONTROLLER_B 6
#define BUTTON_CONTROLLER_A 7

#define BUTTON_NUMPAD_0 0
#define BUTTON_NUMPAD_1 1
#define BUTTON_NUMPAD_2 2
#define BUTTON_NUMPAD_3 3
#define BUTTON_NUMPAD_4 4
#define BUTTON_NUMPAD_5 5
#define BUTTON_NUMPAD_6 6
#define BUTTON_NUMPAD_7 7
#define BUTTON_NUMPAD_8 0
#define BUTTON_NUMPAD_9 1
#define BUTTON_NUMPAD_ENTER 2
#define BUTTON_NUMPAD_PROG 3

#define PROGRAM_MIN 1
#define PROGRAM_MAX 128
#define BASE_PIN_NUM_FOR_BUTTON 2
#define BASE_PIN_NUM_FOR_MODE 10

#define PITCH_MIN 24
#define PITCH_MAX 96

#define CC_MODULATION 1
#define CC_VOLUME 7
#define CC_PAN 10
#define CC_EXPRESSION 11
#define CC_RESONANCE 71
#define CC_RELEASE 72
#define CC_ATACK 73
#define CC_CUTOFF 74
#define CC_REVERB 91
#define CC_CHORUS 93

//#define DEBUG

MIDI_CREATE_DEFAULT_INSTANCE();

bool buttonflag[3][8] = 
{
  {false,false,false,false,false,false,false,false},
  {false,false,false,false,false,false,false,false},
  {false,false,false,false,false,false,false,false}
};

// 楽器
unsigned char program = 1;

unsigned char basePitch = 60;

void setup() {
#ifndef DEBUG
  MIDI.begin(MIDI_CHANNEL_OMNI);
#endif
#ifdef DEBUG
  Serial.begin(9600);
#endif

  // ボタン
  pinMode(2, INPUT_PULLUP);
  pinMode(3, INPUT_PULLUP);
  pinMode(4, INPUT_PULLUP);
  pinMode(5, INPUT_PULLUP);
  pinMode(6, INPUT_PULLUP);
  pinMode(7, INPUT_PULLUP);
  pinMode(8, INPUT_PULLUP);
  pinMode(9, INPUT_PULLUP);

  // モード(GND)
  pinMode(10, OUTPUT);
  pinMode(11, OUTPUT);
  pinMode(12, OUTPUT);
  digitalWrite(10, HIGH);
  digitalWrite(11, HIGH);
  digitalWrite(12, HIGH);

  MIDI.sendProgramChange(program, CH);
}

void loop() {
  unsigned int mode, button;

  int f0 = c(map(analogRead(A0), 695, 710, 0, 127), 0, 127);
  int f1 = c(map(analogRead(A1), 856, 945, 0, 127), 0, 127);
  int f2 = c(map(analogRead(A2), 815, 935, 0, 127), 0, 127);
  int f3 = c(map(analogRead(A3), 785, 920, 0, 127), 0, 127);
#ifdef DEBUG
    Serial.print(f0);
    Serial.print("\t");
    Serial.print(f1);
    Serial.print("\t");
    Serial.print(f2);
    Serial.print("\t");
    Serial.print(f3);
    Serial.print("\t");
#endif
#ifndef DEBUG
  if (f0 > 0) {
    MIDI.sendControlChange(CC_MODULATION, f0, CH);
  }
  if (f1 > 0) {
    MIDI.sendControlChange(CC_EXPRESSION, f1, CH);
  }
  if (f2 > 0) {
    MIDI.sendControlChange(CC_RESONANCE, f2, CH);
  }
  if (f3 > 0) {
    MIDI.sendControlChange(CC_CUTOFF, f3, CH);
  }
/*
  if (f3 > 0 ) {
    MIDI.sendPitchBend(f3, CH);
  }
*/
#endif

  for (mode = 0; mode < NUM_OF_MODE; mode++) {
    digitalWrite(mode + BASE_PIN_NUM_FOR_MODE, LOW);
    for (button = 0; button < NUM_OF_BUTTON; button++) {
      int pinVal = digitalRead(button + BASE_PIN_NUM_FOR_BUTTON);
#ifdef DEBUG
      Serial.print(pinVal);
#endif
      if (pinVal == ON) {
          if (buttonflag[mode][button] == false) {
            if (mode == MODE_CONTROLLER){
              control(mode, button);
            } else {
#ifndef DEBUG
              MIDI.sendNoteOn(basePitch + ((mode - 1) * 8) + button, VELOCITY, CH);
#endif
              buttonflag[mode][button] = true;
            }
          }
      } else {
          if (buttonflag[mode][button] == true) {       
            if ((mode == MODE_NUMPAD1) || (mode == MODE_NUMPAD2)) {
#ifndef DEBUG
              MIDI.sendNoteOff(basePitch + ((mode - 1) * 8) + button, VELOCITY, CH);
#endif
            }
            buttonflag[mode][button] = false;
          }
      }
    }
    digitalWrite(mode + BASE_PIN_NUM_FOR_MODE, HIGH);
  }
#ifdef DEBUG
  Serial.println();
#endif
}

void control(unsigned int mode, unsigned int button) {
  switch (button) {
    case BUTTON_CONTROLLER_DOWN:
      basePitch -= 12;
      if (basePitch < PITCH_MIN) {
        basePitch = PITCH_MIN;
      }
      buttonflag[mode][button] = true;
      break;
    case BUTTON_CONTROLLER_UP:
      basePitch += 12;
      if (basePitch > PITCH_MAX) {
        basePitch = PITCH_MAX;
      }
      buttonflag[mode][button] = true;
      break;
    case BUTTON_CONTROLLER_B:
      program--;
      if (program < PROGRAM_MIN) {
        program = PROGRAM_MIN;
      }
#ifndef DEBUG
      MIDI.sendProgramChange(program, CH);
#endif
      buttonflag[mode][button] = true;
      break;
    case BUTTON_CONTROLLER_A:
      program++;
      if (program > PROGRAM_MAX) {
        program = PROGRAM_MAX;
      }
#ifndef DEBUG
      MIDI.sendProgramChange(program, CH);
#endif
      buttonflag[mode][button] = true;
      break;
  }
  
  return;
}


int c(int val, int min, int max) {
  if (val < min) {
    return min;
  } else if (val > max) {
    return max;
  }
  return val;
}
